module Pages.Services exposing (Model, Msg, page)

import Config.Colour as T exposing (..)
import Config.Format as O exposing (..)
import Config.Identity as I exposing (..)
import Config.Response exposing (..)
import Config.Viewport exposing (..)
import Effect exposing (Effect)
import Element exposing (..)
import Element.Font as F
import Headers.Helpers exposing (headerMaker)
import Headers.Pages.Services exposing (servicesHeader)
import Html.Attributes as H exposing (style)
import Layouts
import Page exposing (Page)
import Route exposing (Route)
import Services.Helpers exposing (..)
import Services.Offerings.DebateAnalysis exposing (..)
import Services.Offerings.DebateTutoring exposing (..)
import Services.Offerings.ElmBuilds exposing (..)
import Services.Offerings.NixBuilds exposing (..)
import Services.Offerings.NutritionScience exposing (..)
import Shared exposing (..)
import View exposing (View)


page : Shared.Model -> Route () -> Page Model Msg
page shared route =
    Page.new
        { init = init
        , update = update
        , subscriptions = subscriptions
        , view = view shared
        }
        |> Page.withLayout toLayout


toLayout : Model -> Layouts.Layout Msg
toLayout model =
    Layouts.Navbar {}



-- INIT


type alias Model =
    {}


init : () -> ( Model, Effect Msg )
init () =
    ( {}
    , Effect.map
        (\_ -> NoOp)
        (Effect.sendCmd resetViewport)
    )



-- UPDATE


type Msg
    = NoOp


update : Msg -> Model -> ( Model, Effect Msg )
update msg model =
    case msg of
        NoOp ->
            ( model
            , Effect.none
            )



-- SUBSCRIPTIONS


subscriptions : Model -> Sub Msg
subscriptions model =
    Sub.none



-- VIEW


view : Shared.Model -> Model -> View Msg
view shared model =
    { title = servicesName
    , attributes = []
    , element = servicesContainer shared.device
    }


servicesContainer : Device -> Element msg
servicesContainer device =
    topLevelContainer (servicesList device)


servicesList : Device -> Element msg
servicesList device =
    column
        (case ( device.class, device.orientation ) of
            ( Phone, Portrait ) ->
                pageListDesktop

            ( Phone, Landscape ) ->
                pageListDesktop

            ( Tablet, Portrait ) ->
                pageListDesktop

            ( Tablet, Landscape ) ->
                pageListDesktop

            ( Desktop, Portrait ) ->
                pageListDesktop

            ( Desktop, Landscape ) ->
                pageListDesktop

            ( BigDesktop, Portrait ) ->
                pageListDesktop

            ( BigDesktop, Landscape ) ->
                pageListDesktop
        )
    <|
        List.concat
            [ List.map headerMaker
                [ servicesHeader ]
            , (case ( device.class, device.orientation ) of
                ( Phone, Portrait ) ->
                    List.map serviceMakerMobile

                ( Phone, Landscape ) ->
                    List.map serviceMakerMobile

                ( Tablet, Portrait ) ->
                    List.map serviceMakerMobile

                ( Tablet, Landscape ) ->
                    List.map serviceMakerMobile

                ( Desktop, Portrait ) ->
                    List.map serviceMaker

                ( Desktop, Landscape ) ->
                    List.map serviceMaker

                ( BigDesktop, Portrait ) ->
                    List.map serviceMaker

                ( BigDesktop, Landscape ) ->
                    List.map serviceMaker
              )
                [ servicesDebateAnalysis
                , servicesDebateTutoring
                , servicesNutritionScience
                , servicesNixBuilds
                , servicesElmBuilds
                ]
            ]
